<?php
//  BARMAZ erp system
//  Copyright (c) BARMAZ Group
//  Web: https://BARMAZ.ru/
//  Commercial license https://BARMAZ.ru/article/litsenzionnoe-soglashenie.html
//  THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//  Revision: 203 (2024-01-07 23:05:31)
// 

//  BARMAZ-ERP
//  Package : module 
//  name : service 
//  Copyright (c) BARMAZ Group
//  Revision: 1.0.1 (2023/09/29)
//  Build: 203 (2024-01-07 23:14:41)
// 


defined('_BARMAZ_VALID') or die("Access denied");

class serviceModeldb extends Model {
	private $_hiddenfiles=array('.','.svn','resources','index.php','index.html','.htaccess','.htpasswd','web.config');
	
	public function backupDB($path,$file){
		$exporter = Exporter::getInstance(false);
		$tables = $exporter->getTablesList();
		$fpsys = fopen($path.$file,"wt");
		//$delimiter=$this->_db->getDelimiter();
		$delimiter=";\n";
		$setnames = "SET NAMES 'UTF8'".$delimiter."\n";
		fputs($fpsys, $setnames, mb_strlen($setnames,DEF_CP));
		foreach ($tables as $t) {
			$expMode = Request::getInt($t.'_mode',0);
			if ($expMode) {
				$createTable = "DROP TABLE IF EXISTS `".$t."`".$delimiter."\n";
				$createTable .= $exporter->getCreateTable($t, $delimiter)."\n";
				fputs($fpsys, $createTable);
				$inserts = $exporter->getInserts($t, $delimiter)."\n";
				fputs($fpsys, $inserts);
			}
		}
		fclose($fpsys);
	}
	public function downloadFile($folder,$file,$href){
		if (in_array($file, $this->_hiddenfiles) || !$folder) Util::redirect($href, Text::_("File not exists")." : ".$file);
		$fullname = $folder.DS.$file;
		if ((file_exists($fullname))&&(is_file($fullname))) {
			header('Content-Description: File Transfer');
			header('Content-Type: application/octet-stream');
			header('Content-Disposition: attachment; filename="'.$file.'"');
			header('Content-Transfer-Encoding: binary');
			header('Expires: 0');
			header('Cache-Control: must-revalidate, post-check=0, pre-check=0');
			header('Pragma: public');
			header("Content-Length: ".filesize($fullname));
			ob_end_clean();
			readfile($fullname);
			Util::halt();
		}
	}
	/**
	 * перечень таблиц в которых могут быть дополнительные поля
	 * возвращает массив из одного поля по именам таблицы 
	 */
	public function getAdditionalTableList()
	{
		$sql="select distinct f_table from #__fields_list";
		$this->_db->setQuery($sql);
		return   $this->_db->loadResultArray();
	}
	/**
	 * список дополнительных полей таблицы
	 * @param unknown $table
	 * @return NULL|unknown[]
	 */
	public function getAdditionalTableFieldList($table)
	{
		$sql="select distinct f_id as id,concat(f_name,'(',f_descr,')') as title  from #__fields_list where f_table='".$table."'";
		$this->_db->setQuery($sql);
		return   $this->_db->loadObjectList('id');
	}
/**
 * 
 * @param int $id  ид доп поля в таблице fields_list 
 * @param string $viewname - контекст view в котором вызвывается процедура( определяем путь хранения файлов) 
 * @return stdClass|boolean  объект со вседениями по доп полю   
 */	
	public function getAddFieldType($id,$m_module=false,$viewname=false,$dopinfo=false)
	{
		$result=false;
		if($id){
			$res=new stdClass();
			$sql="select f.f_id,f.f_name,f.f_descr,t.t_name 
			from #__fields_list as f left join #__fields_type as t on f.f_type=t.t_id
			where f.f_id=".(int)$id;
			$this->_db->setQuery($sql);
			$this->_db->loadObject($res);			
			if($res){
				switch($res->t_name){
					case 'file':
					case 'image':
						if($m_module){
							$res->loadpath=$m_module.DS."_add".DS;
							$res->loadurl=$m_module."/_add/";
						}else{
							$res->loadpath="_add".DS;
							$res->loadurl="/_add/";
							
						}
						// в этот момент мы знаем что у нас для поля есть предустановленные параметры
						// у нас перечень полей для картинок
						if($viewname) {
							$res->loadpath.=$viewname.DS;
							$res->loadurl.=$viewname."/";
						}
						if($dopinfo){
							$res->createWebp=$dopinfo->createWebp;
							$res->d_width=$dopinfo->d_width;
							$res->d_height=$dopinfo->d_height;
							$res->appwm=$dopinfo->appwm;
							$res->d_path_postfix=$dopinfo->d_path_postfix;
						}
						
						
						break;
					default:
						$res->loadpath="";
						$res->loadurl="";
						$res->createWebp=false;
						$res->d_width=0;
						$res->d_height=0;
						$res->appwm=false;
						$res->d_path_postfix="";						
				}
				return $res;
			}
		}
		return $result;
	}
	/**
	 * 
	 * @param string $table - таблица которую рассматриваем 
	 * @param string $fld_name - ключевое поле таблицы
	 * @param boolean $deleteLost - удалять найденный мусор
	 * return object{count,msg};
	 */
	public function checklostDopData($table,$fld_name,$deleteLost=false)
	{
		$sql_count="select count(distinct obj_id) as _cnt from #__".$table."_data where obj_id not in (select ".$fld_name." from #__".$table.")";
		$this->_db->setQuery($sql_count);
		$count=$this->_db->loadResult();
		$msg="";
		if($count>0){
			$msg.=Text::_("Found records for lost positions")." ".$count;
			if($deleteLost)
			{
				$sql_delete="delete from from #__".$table."_data where obj_id not in (select ".$fld_name." from #__".$table.")";
				$this->_db->setQuery($sql_delete);
				
				if($this->_db->query())
				{
					$msg.=" ".Text::_("Founds records deleted");
				}else{
					$msg.=" ".Text::_("Founds records delete not succesfull");
				}
			}
		}else{
			$msg.=Text::_("Records for lost positions not found");
		}
		$return = new stdClass();
		$return->count=$count;
		$return->msg=$msg;
		return $return;		
	}
	
	public function counterRemoveData($table,$fld_name,$str_add_fields){
		$sql_count="select count(obj_id) as _cnt from #__".$table."_data where field_id in (".$str_add_fields.")";
		$this->_db->setQuery($sql_count);
		return $this->_db->loadResult();
	}
	/**
	 * 
	 * @param int $limit - лимит ид основной таблицы приемника 
	 * @param object $inf - объект содеражщий информацию по организации переноса 
	 * @param int $сur_offset - смещение с которого счтать лимит
	 * @return array
	 */
	public function getRecordForRemove($limit,$inf,$сur_offset)
	{
		
		/* два запроса при котором лимитируем не записи доп таблиц а сапи основные записи 
		 * во избежание отрывания и/или неполного заполнения кусков данных    
		 * select m.*  from c_goods_img_data as m where m.field_id in (28,31,32) 
and m.obj_id in (select g.obj_id from (select distinct f.obj_id FROM c_goods_img_data as f order by f.obj_id LIMIT 20 OFFSET 20)as g);

select m.*  from c_goods_img_data as m 
where  m.obj_id in (select g.obj_id from (select distinct f.obj_id FROM c_goods_img_data as f where f.field_id in (28,31,32)  order by f.obj_id LIMIT 20 OFFSET 20)as g);

		 * */
		$arr_prepared_data=array();
		if($inf){
			//Util::showArray($inf);
		// получаем сформированные показатели из таблицы доп полей согласно лимита
			/*$_sql="select *  from #__".$inf->table_src." where field_id in (".$inf->keys_table_src.") 
			order by obj_id LIMIT ".(int)$limit." OFFSET ".(int)$сur_offset;
			*/
			$_sql="select m.*  from #__".$inf->table_src." as m
			where  m.obj_id in 
			(select g.obj_id from (select distinct f.obj_id FROM #__".$inf->table_src." as f 
				where f.field_id in (".$inf->keys_table_src.")
			  order by f.obj_id LIMIT ".(int)$limit." OFFSET ".(int)$сur_offset.")as g)";
			$this->_db->setQuery($_sql);
			//echo $this->_db->getQuery();
			$inf_data=$this->_db->loadObjectList();
			//Util::showArray($inf_data);
			// тут у нас готов набор для переноса данных
			// преобразуем сведения для передачи в обработку
			
			/* массив подготовленных данных
			 * структура 
			 * $arr[id_записи_основной][имя поля основной]=значение
			 * в случае  подготовки данных возможно разумно это готовить тут
			 * и добавить в массив  служебное поле типа подготовка проведена
			 * (как то : 
			 * 	- преобразование картинки к формату webp, 
			 *  - контроль размера изображений
			 *  - смена местоположения файла
			 *  - что-то еще
			 * 
			 */
			
			// 
			if($inf_data&&count($inf_data)>0){
				foreach($inf_data as $k=>$v)
				{
				 $fld_name_dest=$inf->fld_list_d->{$v->field_id};
				 $arr_prepared_data[$v->obj_id][$fld_name_dest]=$v->field_value;
				 // тут надо еще подтянуть тип поля
				}
				
			}
			
		}
		return $arr_prepared_data;
	}
	/**
	 * 
	 * @param int $id - ид основной таблицы для апдейта
	 * @param array $arr_data - данные подготовленные к переносу
	 * @param object $inf - служебные правила переноса
	 * @return boolean
	 */
	public function removeRecordData($id,$arr_data,$inf)
	{  $result_db=false;
	   $delete_old=false; // надо ли удалять исходник при создании webp
	   $_msg=array(); // протокол обработки
	
	   /* основная процедура переноса
	    * 	- преобразование картинки к формату webp,
	    *  - контроль размера изображений
	    *  - смена местоположения файла
	    *  - что-то еще
	    */
	   
	// если у нас нет такой главной записи ничего не случится
	
	/*
	 * [i_med_image]	====> 	detskij_kompleks_jaguar_vm_26_1643625004.jpg (typeof string)
		[i_alt_med_img]	====> 	Детский игровой комплекс «Ягуар» вид сверху превью (typeof string)
		[i_title_med_img]	====> 	Детский игровой комплекс «Ягуар» вид сверху превью (typeof string)
	 * */
	
	$sql="update #__".$inf->table_dst." set ";
	$count_fld=count($arr_data);
	$cntfld=0;
	foreach($arr_data as $kf=>$vf){
		$cntfld++;
		$obj_data=$inf->fld_data->$kf;
		array_push($_msg,Text::_("Manipulation wich records")." ".$kf."=>".$vf);
		if($obj_data->t_name=="image")
		{// у нас будут операции только связанные с изображениями
			
		 // нужно загнать в поле при выборе параметры обработки
			/*[loadpath]	====> 	_add\images\ (typeof string)
			[loadurl]	====> 	/_add/images/ (typeof string)
			[createWebp]	====> 	True (typeof boolean)
			[d_width]	====> 	600 (typeof integer)
			[d_height]	====> 	400 (typeof integer)
			[appwm]	====> 	True (typeof boolean)
			[d_path_postfix]	====> 	i/medium (typeof string)
			*/
		 // сначала получаем файл изображения
			$image=BARMAZ_UF_PATH.$obj_data->loadpath.Files::splitAppendix ( $vf,true );
			array_push($_msg,Text::_("Is image")." ".$image." exists ".(string)file_exists($image));
			$dir = pathinfo($image, PATHINFO_DIRNAME);
			$name = pathinfo($image, PATHINFO_FILENAME);
			$ext= pathinfo($image, PATHINFO_EXTENSION);
			$filename=pathinfo($image, PATHINFO_BASENAME);
			array_push($_msg,Text::_("Is webp`")." ".(string)$obj_data->createWebp." ".$ext);
			
			// физическое новое место назначения файла 
		 // проверка на webp
			if($obj_data->createWebp&&$ext!='webp'){
				// создаем webp из файла
				$res_create=false; // результат создания webp файла
				// подмена файла теперь у нас тут webp рядом лежит
				$image=Files::webpImage($image,80,$delete_old,$res_create);
				if($res_create){
					// тут по идее уже webp
					
					$filename=pathinfo($image, PATHINFO_BASENAME);
					array_push($_msg,Text::_("Create webp")." ".$filename);
				}
			}else{
				array_push($_msg,Text::_("Create webp skipped"));
			}
			
		 // проверка на размеры
			$res_resize=Files::resizeImageNew($image,$image,$obj_data->d_width,$obj_data->d_height);
			if($res_resize) {
				array_push($_msg,Text::_("Resize passed").$obj_data->d_width." x ".$obj_data->d_height);
			}else{
				array_push($_msg,Text::_("Resize error or skipped").$obj_data->d_width." x ".$obj_data->d_height);
			}
		 
		 // проверка на водяной знак
			if($obj_data->appwm){
				// подмена файла теперь у нас тут webp рядом лежит плюс с водяным знаком
				$resappwm=Files::addWaterMark($image,false,$obj_data->d_width,$obj_data->d_height);
				if($resappwm){
					array_push($_msg,Text::_("WaterMark added")." ".$image);
				}else{
					array_push($_msg,Text::_("WaterMark palce failed")." ".$image);
				}
			}
		 // перенос хранения файла(копирование)
			
			
			$_dest_dir=$_dest=BARMAZ_UF_PATH.str_replace("/", DS,$obj_data->d_path_postfix).DS.Files::getAppendix($filename);			
			Files::checkFolder($_dest_dir,true,true);
			$_dest=BARMAZ_UF_PATH.str_replace("/", DS,$obj_data->d_path_postfix).DS.Files::splitAppendix($filename,true );
			array_push($_msg,Text::_("Before copying")." src ".$image." dest  ".$_dest);
			$rescopy=@copy($image,$_dest);
			if($rescopy){
				array_push($_msg,Text::_("Image copied to")." ".$_dest);
			}else{
				$errors= error_get_last();
				array_push($_msg,Text::_("Image copied failed")." src ".$image." dest  ".$_dest." ".$errors['type']." ".$errors['message']);
			}
		 
		 // возможно удаление комплекта доп полей - опция (либо замена ид на 0 с последующей очисткой)
		 
		 // удаление файла из источника	- опция
		 
			$sql.="`".$kf."` = '".$filename."'";
			
		}else{
			array_push($_msg,Text::_("Field type")." ".$obj_data->t_name);
			$sql.="`".$kf."` = '".$vf."'";
			
		}
		if($cntfld<$count_fld) $sql.=",";
	}
	
	$sql.=" where ".$inf->table_dst_key." = ".(int)$id;
	$this->_db->setQuery($sql);
	array_push($_msg,Text::_("Query created")." ".$this->_db->getQuery());
	$result_db=$this->_db->query();
	
	
	$result = new stdClass();
	$result->result_db=$result_db;
	$result->msg=$_msg;
	
	return $result;
	}
}
?>